#!/usr/bin/env python
# -*- coding:UTF-8 -*-
## lifj
## 2018.10.11

'''
process the files generated by calculate_pausing_score.py. There only exists two groups,
the first one is the control groups and the secondary group is the treated groups. e.g -g si-ctrl,si-eIF3e
1) load those files
2) merge those files
3) calculate the mean pausing scores from different replicates
4) output a position weight matrix used for motif logo
input: directory where the puasing score file exist which generated by calculate_pausing_score.py
.....: a data mean score file requires:
1) index: motifs
2) col0: pausing score of control groups
3) col1: pausing score of treat groups
4) col2: ratio values using col1 divided by col0
output:
position weight matrix
'''

import pandas as pd
import numpy as np
import os
import sys
from functools import reduce
from itertools import groupby
from collections import defaultdict
from optparse import OptionParser
from .__init__ import __version__

def create_parse_for_pausing_score_process():
	'''Arguments parse'''
	usage="usage: python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="score_files",help="The input files are pausing scores generated by PausingScore.py. All files are separated by comma. \nOr a data_mean_score with three columns: col0--control pausing score;col1--treated pausing score;col2--ratio. and its index must be motifs")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option('--pausing_score_filter',action="store",type="float",dest="pausing_score_filter",default=10,help="The filter criteria for pausing score on treat samples.default=%default")
	parser.add_option('--ratio_filter',action="store",type='float',dest="ratio_filter",default=2,help="The filter criteria for ratio values. default=%default")
	parser.add_option('--mode',action="store",type='string',dest="input_mode",default='raw',help="The input is the raw density generated by PausingScore.py or a file processed by ProcessPusingScore.py with three columns---[ctrl0 treat1 ratio2]. You can chose raw or ratio. default=%default")
	return parser


def load_pausing_scores(inputFiles,groups,replicates,output_prefix):
	'''according to the files generated by calculate_pausing_score.py, caclulate the mean pausing score of different replicates'''
	label_dict={}
	data_dict=defaultdict(list)
	data_mean_dict=defaultdict()
	replicates_list=','.join(replicates).strip().split(',')
	replicates_files={r:f for r,f in zip(replicates_list,inputFiles)}
	for g,r in zip(groups,replicates):
		label_dict[g]=r.strip().split(',')
	for g in groups:
		for r in label_dict[g]:
			tmp=pd.read_csv(replicates_files[r],sep="\t")
			motifs=tmp.iloc[:,0]
			data_dict[g].append(tmp.iloc[:,1])
	for g in groups:
		data_mean_dict[g]=pd.DataFrame(reduce(lambda x,y: x+y,data_dict[g])/len(data_dict[g]),columns=[g])

	data_mean=pd.concat(data_mean_dict.values(),axis=1)
	data_mean=pd.concat((motifs,data_mean),axis=1)
	data_mean.to_csv(output_prefix+"_mean_pausing_score.txt",sep="\t",index=0)
	return data_mean

def process_mean_data(data_mean,output_prefix):
	data_mean_score=data_mean.groupby('motif').sum()
	data_mean_score=data_mean_score.sort_values(by=data_mean_score.columns[1],ascending=False)
	## filtering zeros
	data_mean_score=data_mean_score[data_mean_score>0]
	data_mean_score=data_mean_score.dropna()
	## filtering motifs whose length less than 5 (EPA + 2 spaces)
	motifs=[motif for motif in data_mean_score.index  if len(motif)==3]
	data_mean_score=data_mean_score.loc[motifs,:]
	## calculate the weight
	data_mean_score['ratio']=data_mean_score.iloc[:,1]/data_mean_score.iloc[:,0]
	data_mean_score.to_csv(output_prefix+"_mean_pausing_score_filtred.txt",sep="\t")

	return data_mean_score



def calculate_pwm(data_mean_score,output_prefix,ratio_filter,pausing_score_filter):
	'''according to the mean pausing score and the motifs, calculate the position weight martix'''

	data_mean_score_filtered=data_mean_score[(data_mean_score['ratio']>=ratio_filter) & (data_mean_score.iloc[:,1]>=pausing_score_filter)]
	if data_mean_score_filtered.shape[0] <1:
		raise IOError("There is no AAA motifs left for too strict filter parameters! Please reset your filter parameters!")
	data_mean_score_filtered.to_csv(output_prefix+"_motifs_used_for_pwm.txt",sep="\t")

	## PWM
	PWM=pd.DataFrame()
	PWM['motif']=data_mean_score_filtered.index
	PWM['weight']=data_mean_score_filtered['ratio'].values
	# PWM['E'],PWM['P'],PWM['A']=PWM['motif'].str.split(' ').str
	PWM['E'],PWM['P'],PWM['A']=PWM['motif'].str
	pwm_E=PWM[['E','weight']].groupby('E').sum()
	pwm_P=PWM[['P','weight']].groupby('P').sum()
	pwm_A=PWM[['A','weight']].groupby('A').sum()
	pwm=pd.merge(pwm_E,pwm_P,how='left',left_index=True,right_index=True)
	pwm=pd.merge(pwm,pwm_A,how='left',left_index=True,right_index=True)
	pwm.columns=['E','P','A']
	pwm=pwm.fillna(0)
	pwm['E']=pwm['E']/sum(pwm['E'])
	pwm['P']=pwm['P']/sum(pwm['P'])
	pwm['A']=pwm['A']/sum(pwm['A'])
	pwm=pwm.T
	pwm=pwm.reset_index(drop=True)
	pwm.to_csv(output_prefix+"_pwm.txt",sep="\t")


def parse_args_for_pwm_calculation():
	'''Arguments parse'''
	parser=create_parse_for_pausing_score_process()
	(options,args)=parser.parse_args()
	(inputFiles,output_prefix,groups,replicates,pausing_score_filter,ratio_filter)=(options.score_files.strip().split(","),options.output_prefix,options.group_name.strip().split(','),options.replicate_name.strip().split('__'),options.pausing_score_filter,options.ratio_filter)
	replicates_list=','.join(replicates).strip().split(',')
	if options.input_mode == 'raw':
		if len(inputFiles)!=len(replicates_list):
			raise IOError("The number of input files is not consistent with numbers of replicates, please check it!")
		print("Start calculating the mean pausing score.",file=sys.stderr)
		data_mean=load_pausing_scores(inputFiles,groups,replicates,output_prefix)
		data_mean_score=process_mean_data(data_mean,output_prefix)
		print("Start calculating the position weight matrix...",file=sys.stderr)
		calculate_pwm(data_mean_score,output_prefix,ratio_filter,pausing_score_filter)
		print("Finish the step of pwm calculation",file=sys.stderr)
	elif options.input_mode == 'ratio':
		data_mean_score=pd.read_csv(inputFiles[0],sep="\t",index_col=0)
		print("Start calculating the position weight matrix...",file=sys.stderr)
		calculate_pwm(data_mean_score,output_prefix,ratio_filter,pausing_score_filter)
		print("Finish the step of pwm calculation",file=sys.stderr)
	else:
		raise IOError("Please input a correct --mode parameter.[raw or ratio]")

def main():
	parse_args_for_pwm_calculation()

if __name__ == "__main__":
	main()







#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-21 20:55:35
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 17:01:33
@Description: This script is used for statistic GC contents of specific transcripts or GC contents on different reading frames of given transcripts.

Notice:
1) input any DNA or RNA sequences, and set --mode noraml. This will output GC contents of sequences you input.
2) input cds sequences and set --mode frames. This will output GC contents on different reading frames for those cds sequences.
'''

import sys
from itertools import groupby
from optparse import OptionParser
from .__init__ import __version__

def create_parser_for_GC_content():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="sequences",
			help="Input file(s) in fasta format.")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("--mode",action="store",type="string",dest="mode",default="normal",
			help="The type of GC content you want to statistic. Either the normal type or GC content from each reading frame. [normal or frames]. defaul=%default")
	return parser

def fastaIter(transcriptFile):
	'''
	This function is used to get a dict of transcript sequence
	'''
	fastaDict={}
	f=open(transcriptFile,'r')
	faiter=(x[1] for x in groupby(f,lambda line: line.strip()[0]==">")) ## groupby returns a tuple (key, group)
	for header in faiter:
		geneName=header.__next__().strip(">").split(" ")[0]
		seq=''.join(s.strip() for s in faiter.__next__())
		fastaDict[geneName]=seq
	return fastaDict

def write_GC_content(sequences,output_prefix):
    sequences_dict=fastaIter(sequences)
    with open(output_prefix+"_GC_content.txt","w") as f:
        f.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" %('transcripts','A','T/U','C','G','GC','total','GC%'))
        for trans in sequences_dict.keys():
            seq=sequences_dict[trans]
            A,T_or_U,C,G,GC,total,GC_pct=GC_content_for_single_sequence(seq)
            f.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" %(trans,str(A),str(T_or_U),str(C),str(G),str(GC),str(total),str(GC_pct)))
def GC_content_for_single_sequence(seq):
    A=seq.count('A')
    G=seq.count('G')
    C=seq.count('C')
    T_or_U=len(seq)-A-G-C
    GC=G+C
    total=len(seq)
    if total==0:
        raise IOError("There is empty sequence in your input file.")
    GC_pct=GC/total
    return A,T_or_U,C,G,GC,total,GC_pct

def GC_content_for_different_frame(sequences,output_prefix):
    ''' Notice: the input file must be cds sequences with fasta format.'''
    sequences_dict=fastaIter(sequences)
    i=0
    with open(output_prefix+"_GC_content_frames.txt","w") as f:
        f.write("%s\t%s\t%s\t%s\t%s\n" %('transcripts','frame0','frame1','frame2','frameSum'))
        for trans in sequences_dict.keys():
            seq=sequences_dict[trans]
            if len(seq) % 3!=0:
                seq = seq[:-(len(seq) % 3)]
                i+=1
            frame0=[seq[i] for i in range(0,len(seq),3)]
            frame1=[seq[i] for i in range(1,len(seq),3)]
            frame2=[seq[i] for i in range(2,len(seq),3)]
            frame0_GC_pct=GC_content_for_single_sequence(frame0)[-1]
            frame1_GC_pct=GC_content_for_single_sequence(frame1)[-1]
            frame2_GC_pct=GC_content_for_single_sequence(frame2)[-1]
            frameSum=GC_content_for_single_sequence(seq)[-1]
            f.write("%s\t%s\t%s\t%s\t%s\n" %(trans,frame0_GC_pct,frame1_GC_pct,frame2_GC_pct,frameSum))

def main():
    parser=create_parser_for_GC_content()
    (options,args)=parser.parse_args()
    if not options.sequences or not options.output_prefix:
        raise IOError("Please your input sequences and prefix of your output files.")
    if options.mode == 'normal':
        print("Start statistic GC content...",file=sys.stderr)
        write_GC_content(options.sequences,options.output_prefix)
        print("Finish the step of GC content!",file=sys.stderr)
    elif options.mode == 'frames':
        print("Start statistic GC content...",file=sys.stderr)
        GC_content_for_different_frame(options.sequences,options.output_prefix)
        print("Finish the step of GC content!",file=sys.stderr)
    else:
        raise IOError("Please reset your --mode parameter [normal/frames]")

if __name__=="__main__":
    main()





#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-22 11:42:42
@LastEditors: Li Fajin
@LastEditTime: 2019-09-04 20:53:15
@Description: This script is used for statistic the length distribution of sequence reads based on a fastq file.
'''



import sys
import numpy as np
from itertools import groupby
from collections import defaultdict
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
from optparse import OptionParser
from .__init__ import __version__


def create_parse_for_plot_reads_length():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option('-i','--input',action="store",type="string",dest="fastqFile",help="Sequence fastq file.[required]")
	parser.add_option('-o',"--otput_prefix",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	return parser

def fq2seqDict(fqFile):
	'''
	This function is used to get a dict of transcript sequence based on fastq file
	'''
	fastaDict={}
	f=open(fqFile,'r')
	faiter=(x[1] for x in groupby(f,lambda line: line[0]=="@")) ## groupby returns a tuple (key, group)
	for header in faiter:
		read_name=header.__next__().strip("@").split(" ")[0]
		seq='\t'.join(s.strip() for s in faiter.__next__()).strip().split("\t")[0]
		fastaDict[read_name]=seq
	return fastaDict

def get_read_length(fqFile,output_prefix):
	'''
	This function is used for getting length of all reads from a sequence fastq file.
	'''
	lengths_list=[]
	lengths_dict=defaultdict(int)
	zeros=0
	read_sequences=fq2seqDict(fqFile)
	with open(output_prefix+"_reads_length.txt",'w') as f:
		f.write("%s\t%s\n" %("read_name","read_length"))
		for read in read_sequences.keys():
			read_seq=read_sequences[read]
			read_length=len(read_seq)
			if read_length == 0:
				zeros+=1
			lengths_dict[read_length]+=1
			lengths_list.append(read_length)
			f.write("%s\t%s\n" % (str(read),str(read_length)))
	lengths_list=np.array(lengths_list)
	print("The total reads number is: " + str(len(lengths_list)),file=sys.stderr)
	print("The zero length number is: " + str(zeros),file=sys.stderr)
	print("The mean of reads length is: " + str(lengths_list.mean()),file=sys.stderr)
	print("The sd of reads length is: " + str(lengths_list.std()),file=sys.stderr)
	return lengths_list,lengths_dict

def plot_reads_length(lengths_dict,output_prefix,text_font={"size":20,"family":"Arial","weight":"bold"}):
	'''
	plot length distribution
	'''
	labels=lengths_dict.keys()
	values=lengths_dict.values()
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	plt.bar(labels,values,color="b",width=0.5,alpha=0.9)
	ax.spines["top"].set_linewidth(2)
	ax.spines["right"].set_linewidth(2)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.set_xlabel("Length of reads",fontdict=text_font)
	ax.set_ylabel("Read Counts",fontdict=text_font)
	# ax.set_xticks(np.arange(0,50,5))
	# ax.set_xticklabels((np.arange(0,50,5)))
	ax.tick_params(which="both",width=2,labelsize=10)
	plt.tight_layout()
	plt.savefig(output_prefix+"_reads_length.pdf")
	plt.close()

def main():
	parser=create_parse_for_plot_reads_length()
	(options,args)=parser.parse_args()
	if not options.fastqFile or not options.output_prefix:
		raise IOError("Please reset your parameters!")
	print("Start the step of length statistics...",file=sys.stderr)
	lengths_list,lengths_dict=get_read_length(options.fastqFile,options.output_prefix)
	plot_reads_length(lengths_dict,options.output_prefix,text_font={"size":15,"family":"Arial","weight":"bold"})
	print("Finish the step of length statistics!",file=sys.stderr)

if __name__=="__main__":
	main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-16 17:01:32
@LastEditors: Li Fajin
@LastEditTime: 2019-09-01 21:42:17
@Description: This script is used for cAI plot.
'''

import numpy as np
import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from .__init__ import __version__



def create_parser_for_cAI_plot():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="density_file",help="Input file in txt format.And the files has three columns; column 1: sample;columns 2: start_density; column 3: stop_density")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon", help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="Control the mode for plot.[all or single]. default=%default")
	parser.add_option("--axvline",action="store",type="float",dest="axvline",default=None,help="Position to plot vetical line")
	parser.add_option("--start",action="store",type="int",dest="start_position",default=5,help="The start position need to be averaged.default=%default")
	parser.add_option("--window",action="store",type="int",dest="window",default=7,help="The length of silde window. ddefault=%default")
	parser.add_option("--step",action="store",type='int',dest="step",default=1,help="The step length of slide window. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The min of ylim. default=%default")
	return parser

def plot_all_density(data,samples,type,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,inOutFomat,axvline,ymin,ymax,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
		'''plot the cAI'''
		plt.rc('font',weight='bold')
		fig=plt.figure(figsize=(16,8))
		ax=fig.add_subplot(111)
		winLen=in_regionLengthParma+in_extendRegionLengthParma+1
		# font={"size":20,"family":"Arial","weight":"bold"}
		# # colors="bgrcmykwbgrcmykw"
		if len(samples) <=8:
			colors=["b","orangered","green","c","m","y","k","w"]
		else:
			colors=sns.color_palette('husl',len(samples))
		for i in np.arange(len(samples)):
			if type=="start codon":
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen,50))
				ax.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
				if axvline:
					ax.axvline(axvline,color="r",dashes=(3,2))
				else:
					pass
			else:
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen,50))
				ax.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))
				if axvline:
					ax.axvline(winLen-axvline,color="r",dashes=(3,2))
				else:
					pass
		ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
		ax.set_ylabel("Local cAI",fontdict=text_font)
		#	 ax.set_title("Ribosome footprint density profiles",fontdict=text_font)
		ax.spines["top"].set_visible(False)
		ax.spines["right"].set_visible(False)
		ax.spines["bottom"].set_linewidth(2)
		ax.spines["left"].set_linewidth(2)
		ax.tick_params(which="both",width=2,labelsize=20)
		if not ymin and not ymax:
			pass
		elif not ymin and ymax:
			ax.set_ylim(0,ymax)
		elif ymin and not ymax:
			raise IOError("Please offer the ymax parameter as well!")
		elif ymin and ymax:
			ax.set_ylim(ymin,ymax)
		else:
			raise IOError("Please enter correct ymin and ymax parameters!")
		plt.legend(loc="best",prop=legend_font)
		plt.savefig(inOutPrefix+"_"+type+"."+inOutFomat,format=inOutFomat)
		plt.close()
		# plt.show()

def plot_density_for_each_sample(data,samples,type,in_regionLengthParma,in_extendRegionLengthParma,inOutFomat,ymin,ymax,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
		'''plot the cAI'''
		plt.rc('font',weight='bold')
		winLen=in_regionLengthParma+in_extendRegionLengthParma+1
		# font={"size":20,"family":"Arial","weight":"bold"}
		# # colors="bgrcmykwbgrcmykw"
		colors=["b","orangered","green","c","m","y","k","w"]
		for i in np.arange(len(samples)):
			if type=="start codon":
				fig=plt.figure(figsize=(16,8))
				ax=fig.add_subplot(111)
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen,50))
				ax.set_xticklabels((np.arange(0,winLen,50)-in_extendRegionLengthParma))
				ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
				ax.set_ylabel("Local cAI",fontdict=text_font)
				ax.spines["top"].set_visible(False)
				ax.spines["right"].set_visible(False)
				ax.spines["bottom"].set_linewidth(2)
				ax.spines["left"].set_linewidth(2)
				ax.tick_params(which="both",width=2,labelsize=20)
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
				plt.legend(loc="best",prop=legend_font)
				plt.savefig(str(samples[i])+"_cAI_start_codon"+"."+inOutFomat,format=inOutFomat)
				plt.close()

			else:
				fig=plt.figure(figsize=(16,8))
				ax=fig.add_subplot(111)
				plt.plot(np.arange(0,winLen),data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2],color=colors[i],label=samples[i],linewidth=1)
				ax.set_xticks(np.arange(0,winLen,50))
				ax.set_xticklabels((np.arange(0,winLen,50)-in_regionLengthParma))
				ax.set_xlabel("Distance from "+type+" (codon)",fontdict=text_font)
				ax.set_ylabel("Local cAI",fontdict=text_font)
				ax.spines["top"].set_visible(False)
				ax.spines["right"].set_visible(False)
				ax.spines["bottom"].set_linewidth(2)
				ax.spines["left"].set_linewidth(2)
				ax.tick_params(which="both",width=2,labelsize=20)
				if not ymin and not ymax:
					pass
				elif not ymin and ymax:
					ax.set_ylim(0,ymax)
				elif ymin and not ymax:
					raise IOError("Please offer the ymax parameter as well!")
				elif ymin and ymax:
					ax.set_ylim(ymin,ymax)
				else:
					raise IOError("Please enter correct ymin and ymax parameters!")
				plt.legend(loc="best",prop=legend_font)
				plt.savefig(str(samples[i])+"_cAI_stop_codon"+"."+inOutFomat,format=inOutFomat)
				plt.close()
def slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,inOutPrefix,start,window,step):
	start_average=[]
	stop_average=[]
	label=[]
	winLen=in_regionLengthParma+in_extendRegionLengthParma+1
	for i in np.arange(len(samples)):
			tmp1_data=np.zeros(winLen)
			tmp1_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][0:int(start)]
			tmp1_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][-int(start):]
			for j in np.arange(start,winLen-start,step):
				tmp1_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,1][(j-int((window-1)/2)):(j+int((window-1)/2))])
			start_average.extend(tmp1_data)
			label.extend([samples[i]]*winLen)

			tmp2_data=np.zeros(winLen)
			tmp2_data[0:int(start)]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][0:int(start)]
			tmp2_data[-int(start):]+=data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][-int(start):]
			for j in np.arange(start,winLen-start,step):
				tmp2_data[j]+=np.mean(data.iloc[np.where(data.iloc[:,0]==samples[i])].iloc[:,2][(j-int((window-1)/2)):(j+int((window-1)/2))])
			stop_average.extend(tmp2_data)

	data_average=pd.DataFrame([label,start_average,stop_average],index=['sample','start_density','stop_density'])
	data_average=data_average.T
	data_average.to_csv(inOutPrefix+"_average_cAI.txt",sep="\t",index=0)
	return data_average

def main():
		parsed=create_parser_for_cAI_plot()
		(options,args)=parsed.parse_args()
		(data,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,mode,start,window,step,ymin,ymax,axvline)=(options.density_file,options.downstream_codon,options.upstream_codon,
		options.output_prefix,options.output_format,options.mode,options.start_position,options.window,options.step,options.ymin,options.ymax,options.axvline)
		if window%2 == 0:
			raise IOError("Please reset your --window parameter. It must be a odd number.")
		if (start-1) < (window-1)/2:
			raise IOError("Please reset your --step and --window parameters. The (window-1)/2 must be less than start-1")
		print("your input file is: "+str(data),file=sys.stderr)
		data=pd.read_csv(data,sep="\t")
		samples=np.unique(data.iloc[:,0])
		text_font={"size":30,"family":"Arial","weight":"bold"}
		legend_font={"size":30,"family":"Arial","weight":"bold"}
		data_average=slide_window_average(data,samples,in_regionLengthParma,in_extendRegionLengthParma,output_prefix,start,window,step)
		if mode == 'all':
			plot_all_density(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,axvline,ymin,ymax,text_font=text_font,legend_font=legend_font)
			plot_all_density(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix,output_format,axvline,ymin,ymax,text_font=text_font,legend_font=legend_font)
			plot_all_density(data_average,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,axvline,ymin,ymax,text_font=text_font,legend_font=legend_font)
			plot_all_density(data_average,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_prefix+"_average",output_format,axvline,ymin,ymax,text_font=text_font,legend_font=legend_font)
			print("finished plot the ribosome footprint density",file=sys.stderr)
		elif mode == 'single':
			plot_density_for_each_sample(data,samples,"start codon",in_regionLengthParma,in_extendRegionLengthParma,output_format,ymin,ymax,text_font=text_font,legend_font=legend_font)
			plot_density_for_each_sample(data,samples,"stop codon",in_regionLengthParma,in_extendRegionLengthParma,output_format,ymin,ymax,text_font=text_font,legend_font=legend_font)
			print("finished plot the ribosome footprint density",file=sys.stderr)
		else:
			raise IOError("please reset your --mode parameter [all or single]")

if __name__ =="__main__":
			main()




#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-19 23:01:51
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 17:08:08
@Description: This script used for calculating ribosome density at each kind of amino acid or codon.
And the ribosome density means counts/sequence_depth.
input:
	the input are just the same as MetageneAnalysis.py but with longest.transcripts.cds.sequence.fa file input also which could be generated by GetProteinCodingSequence.py
output:
	if -u and -d not set: there is only one output file with Row x Col where Row is the numer of  codon kinds and the Col is the number of columns which usually are just like:

	AA      codon   si_ctrl_1_80S   si_ctrl_2_80S   si_ctrl_3_80S   si_3e_1_80S     si_3e_2_80S     si_3e_3_80S
	K       AAA     0.01689667937572358     0.01592644427516077     0.015870156244881917    0.013608564821264385    0.013323948113603084    0.122
	N       AAC     0.020736512248088333    0.02236358252621296     0.0213430945439855      0.02247569028447243     0.02225038863195103     0.234
	K       AAG     0.03195537918496984     0.03208825472187564     0.032316267592562266    0.02973099105625366     0.0282806341957415      0.1212
	N       AAT     0.019226280160431212    0.019885910677272484    0.020112024632316816    0.019848987211278594    0.019645264495487997    0.123

	where the first column is AA, the second column is codon, and the rest of other columns are samples.
	if -u and -d are set: there are two output files with the same format as what we talked above.
	the first one file is the density on the specific region and the other file is the density out of the specific region.
'''


from .FunctionDefinition import *
import re
from collections import defaultdict
from itertools import groupby
import Bio.Data.CodonTable as ct

def codon_density(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,inCDS_countsFilterParma,inCDS_lengthFilterParma,transcriptFasta,left_pos,right_pos,mode):
	'''output the density of each class of codon'''
	transcript_sequence=fastaIter(transcriptFasta)
	passTransSet=set()
	filter_1=0
	filter_2=0
	filter_3=0
	if left_pos and (not right_pos):
		raise IOError("Please input the right position match with the left position!")
	if (not left_pos) and right_pos:
		raise IOError("Please input the left position match with the right position!")
	if (not left_pos) and (not right_pos):
		pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
		pysamFile_trans=pysamFile.references
		in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
		all_codon_density=defaultdict(list)
		all_counts=0
		for trans in in_selectTrans:
			leftCoor =int(in_startCodonCoorDict[trans])-1
			rightCoor=int(in_stopCodonCoorDict[trans])-3
			(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			all_counts+=total_reads
		for trans in in_selectTrans:
			leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
			rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
			CDS_seq=transcript_sequence[trans][:-3] ## excluding the stop codon
			# print(CDS_seq)
			if len(CDS_seq) % 3 != 0:
				filter_1+=1
				continue
			if len(CDS_seq) < inCDS_lengthFilterParma:
				filter_2+=1
				continue
			codon_seq=[]
			for i in np.arange(0,len(CDS_seq),3):
				codon=CDS_seq[i:(i+3)]
				codon_seq.append(codon)
			# print(codon_seq)
			(read_counts,read_counts_frameSum,trans_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			cds_reads_norm = 10**9*(cds_reads/(all_counts*len(read_counts_frameSum)*3))
			if mode == 'RPKM':
				if cds_reads_norm < inCDS_countsFilterParma:
					filter_3+=1
					continue
			if mode == 'counts':
				if cds_reads < inCDS_countsFilterParma:
					filter_3+=1
					continue
			all_read_counts_frame=np.array(read_counts_frameSum)
			codon_seq=np.array(codon_seq)
			# all_densityPerTrans=10**6*(all_read_counts_frame/all_counts)
			all_densityPerTrans=all_read_counts_frame/all_counts
			# print(trans+"\t"+"codon: "+str(len(codon_seq))+"\t"+"counts: "+str(len(all_densityPerTrans))+"\t" + "TAG: " + str(list(codon_seq).count('TAG'))+"\t" + "TGA: " + str(list(codon_seq).count('TGA'))+"\t" + "TAA: " + str(list(codon_seq).count('TAA')),file=sys.stderr)
			for k,v in zip(codon_seq,all_densityPerTrans):
				all_codon_density[k].append(v)
			passTransSet.add(trans)
		pysamFile.close()
		for k,v in all_codon_density.items():
			all_codon_density[k]=np.sum(v)
		print("The number of transcripts filtered by filter1 (len(CDS_seq)%3!=0) is : " + str(filter_1),file=sys.stderr)
		print("The number of transcripts filtered by filter2 (len(CDS_seq)<inCDS_lengthFilterParma) is :" + str(filter_2),file=sys.stderr)
		print("The number of transcripts filtered by filter3 (cds_reads < inCDS_countsFilterParma) is : "+str(filter_3),file=sys.stderr)
		print("The number of transcripts used for following analysis is: " + str(len(passTransSet)),file=sys.stderr)
		return all_codon_density
	if left_pos and right_pos:
		if right_pos <= left_pos:
			raise IOError("The right position must be larger than the left position!")
		pysamFile=pysam.AlignmentFile(in_bamFile,"rb")
		pysamFile_trans=pysamFile.references
		in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
		specific_range_codon_density = defaultdict(list)
		the_rest_range_codon_density=defaultdict(list)
		all_counts=0
		for trans in in_selectTrans:
			leftCoor =int(in_startCodonCoorDict[trans])-1
			rightCoor=int(in_stopCodonCoorDict[trans])-3
			(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			all_counts+=total_reads
		for trans in in_selectTrans:
			if (int(right_pos)-int(left_pos)) > in_transLengthDict[trans]:
				raise IOError("The length of the range you chose is larger than some transcript! please reset your parameters!")
			if right_pos > in_transLengthDict[trans]:
				raise IOError("The right position is larger than the length of some transcrip! please reset your parameters!")
			leftCoor =int(in_startCodonCoorDict[trans])-1 #the first base of start codon 0-base
			rightCoor=int(in_stopCodonCoorDict[trans])-3 #the first base of stop codon 0-base
			CDS_seq=transcript_sequence[trans][:-3]
			if len(CDS_seq) % 3 != 0:
				filter_1+=1
				continue
			if len(CDS_seq) < inCDS_lengthFilterParma:
				filter_2+=1
				continue
			codon_seq=[]
			for i in np.arange(0,len(CDS_seq),3):
				codon=CDS_seq[i:(i+3)]
				codon_seq.append(codon)
			(read_counts,read_counts_frameSum,trans_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
			cds_reads_norm = 10**9*(cds_reads/(all_counts*len(read_counts_frameSum)*3))
			if mode == 'RPKM':
				if cds_reads_norm < inCDS_countsFilterParma:
					filter_3+=1
					continue
			if mode == 'counts':
				if cds_reads < inCDS_countsFilterParma:
					filter_3+=1
					continue
			specific_range_read_counts_frame=np.array(read_counts_frameSum[int(left_pos-1):right_pos])
			the_rest_range_read_counts_frame=np.array(read_counts_frameSum[right_pos:])
			codon_seq=np.array(codon_seq)
			specific_range_codon_seq=codon_seq[int(left_pos-1):right_pos]
			the_rest_range_codon_seq=codon_seq[right_pos:]
			specific_range_densityPerTrans=specific_range_read_counts_frame/all_counts
			the_rest_range_densityPerTrans=the_rest_range_read_counts_frame/all_counts
			# specific_range_densityPerTrans=10**6*(specific_range_read_counts_frame/all_counts)
			# the_rest_range_densityPerTrans=10**6*(the_rest_range_read_counts_frame/all_counts)
			for k,v in zip(specific_range_codon_seq,specific_range_densityPerTrans):
				specific_range_codon_density[k].append(v)
			for k,v in zip(the_rest_range_codon_seq,the_rest_range_densityPerTrans):
				the_rest_range_codon_density[k].append(v)
			passTransSet.add(trans)
		pysamFile.close()
		for k,v in specific_range_codon_density.items():
			specific_range_codon_density[k]=np.sum(v)
		for k,v in the_rest_range_codon_density.items():
			the_rest_range_codon_density[k]=np.sum(v)
		print("The number of transcripts filtered by filter1 (len(CDS_seq)%3!=0) is : " + str(filter_1),file=sys.stderr)
		print("The number of transcripts filtered by filter2 (len(CDS_seq)<inCDS_lengthFilterParma) is :" + str(filter_2),file=sys.stderr)
		print("The number of transcripts filtered by filter3 (cds_reads < inCDS_countsFilterParma) is : "+str(filter_3),file=sys.stderr)
		print("The number of transcripts used for following analysis is: " + str(len(passTransSet)),file=sys.stderr)
		return specific_range_codon_density,the_rest_range_codon_density

def codon2AA(table=1):
	''' table=1 is a standard codon table'''
	codon2AA_dict={k: v.forward_table for k, v in ct.unambiguous_dna_by_id.items()}
	return codon2AA_dict[table]
def get_stop_codons(table=1):
	stop_codons_dict={k: v.stop_codons for k, v in ct.unambiguous_dna_by_id.items()}
	return stop_codons_dict[table]
def shapeData(data,table=1):
	''' use standard codon table'''
	Codon_table=codon2AA(table=table)
	stopCodons=get_stop_codons(table=table)
	for stop_codon in stopCodons:
		Codon_table[stop_codon]="*"
	data=data.reset_index()
	data=data.rename(columns={'index':'codon'})
	codon_list=data.iloc[:,0].values
	AA_list=[]
	for codon in codon_list:
		AA_list.append(Codon_table[codon])
	AA_list=pd.DataFrame(AA_list,columns=['AA'])
	data=pd.concat((AA_list,data),axis=1)
	return data


def write_specific_range_codon_density(inBamAttr,outFile,table=1):
	specific_data=[]
	rest_data=[]
	data_index=[]
	for bms in inBamAttr:
		specific_codon_density=bms.specific_range_codon_density
		rest_codon_density=bms.the_rest_range_codon_density
		sample=bms.bamLegend
		specific_data.append(specific_codon_density)
		rest_data.append(rest_codon_density)
		data_index.append(sample)
	specific_data=pd.DataFrame(specific_data,index=data_index)
	rest_data=pd.DataFrame(rest_data,index=data_index)
	specific_data=specific_data.T
	rest_data=rest_data.T
	specific_data=shapeData(specific_data,table=table)
	rest_data=shapeData(rest_data,table=table)
	specific_data.to_csv(outFile+"_specific_range_codon_density.txt",sep='\t',index=0)
	rest_data.to_csv(outFile+"_rest_range_codon_density.txt",sep="\t",index=0)

def write_all_codon_density(inBamAttr,outFile,table=1):
	all_data=[]
	data_index=[]
	for bms in inBamAttr:
		all_codon_density=bms.all_codon_density
		sample=bms.bamLegend
		all_data.append(all_codon_density)
		data_index.append(sample)
	all_data=pd.DataFrame(all_data,index=data_index)
	all_data=all_data.T
	all_data=shapeData(all_data,table=table)
	all_data.to_csv(outFile+"_all_codon_density.txt",sep="\t",index=0)

def parse_args_for_codon_density_calculation():
	parsed=creat_parser_for_calculation_of_codon_density()
	(options,args)=parsed.parse_args()
	(output_prefix, min_cds_codon,min_cds_counts,left_position, right_position, transcript_fasta,mode,table) = (options.output_prefix,options.min_cds_codon,
	options.min_cds_counts,options.upstream_position,options.downstream_position,options.transcript_fasta,options.mode,options.geneticCode)
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans

	if left_position and right_position:
		for bamfs in bam_attr:
			(bamfs.specific_range_codon_density,bamfs.the_rest_range_codon_density)=codon_density(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,
			stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,min_cds_counts,min_cds_codon,transcript_fasta,left_position,right_position,mode)
		write_specific_range_codon_density(bam_attr,output_prefix,table=table)
		print("Finish the step of codon density calculation",file=sys.stderr)
	if (not left_position) and (not right_position):
		for bamfs in bam_attr:
			bamfs.all_codon_density=codon_density(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,
			stopCodonCoorDict,bamfs.bamLen,bamfs.bamOffset,min_cds_counts,min_cds_codon,transcript_fasta,left_position,right_position,mode)
		write_all_codon_density(bam_attr,output_prefix,table=table)
		print("Finish the step of codon density calculation",file=sys.stderr)


def main():
	"""main program"""
	parse_args_for_codon_density_calculation()

if __name__ == "__main__":
		main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-14 11:32:34
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 16:43:36
@Description: This script is used for plot metagene results for the whole transcripts
			usage: python PlotMetageneAnalysisForTheWholeRegions.py -i test_scaled_dataframe.txt -o test -g si-Ctrl,si-eIF3e -r si_ctrl_1_80S,si_ctrl_2_80S,si_ctrl_3_80S__si_3e_1_80S,si_3e_2_80S,si_3e_3_80S -f pdf -b 15,90,60 --mode all
			input:
			1) metagene results generated by MetageneAnalysisForTheWholeRegions.py
			output (selective):
			1) plots for all samples
			2) plots for mean samples
'''


import numpy as np
import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from functools import reduce
from itertools import chain
from collections import defaultdict
from .__init__ import __version__



def create_parser_for_the_whole_metagene_plot():
	'''argument parser'''
	usage="usage: python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="density_file",help="Input file in txt format.And the files have N columns, meaning N samples and have total bins rows. [FiveUTR+CDS+ThreeUTR]")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option("-b","--bins",action='store',type='string',dest="bins",default='15,90,60',help="Bins to scale the transcript length.e.g.'15,90,60'. bins must be separated by comma, namely '5UTRBins,CDSBins,3UTRBins'. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The min of ylim. default=%default")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="plot all samples or just mean samples [all or mean].If choose 'all',output all samples as well as mean samples, else just mean samples.default=%default")
	parser.add_option("--xlabel-loc",action="store",type="float",dest="xlabelLoc",default=None,help="Location of xlabel. Used to control the yaxis location of xlabel. default=%default")

	return parser

def lengths_offsets_split(value):
		''' Split the given comma separated values to multiple integer values'''
		values=[]
		for item in value.split(','):
				item=int(item)
				values.append(item)
		return values

def plot_read_coverage_distribution(data,samples,bins,inOutPrefix,inOutFomat,ymin,ymax,xlabelLoc,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	'''plot the density dsitribution'''
	plt.rc('font',weight='bold')
	fig=plt.figure(figsize=(10,6))
	ax=fig.add_subplot(111)
	bins_vector=lengths_offsets_split(bins)
	winLen=np.sum(bins_vector)
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	for i in np.arange(len(samples)):
		plt.plot(np.arange(0,winLen),data.loc[:,samples[i]],color=colors[i],label=samples[i],linewidth=2)
	ax.set_xticks([])
	ax.axvline(int(bins_vector[0]),color='gray',dashes=(2,3),clip_on=False,linewidth=2)
	ax.axvline(int(bins_vector[0])+int(bins_vector[1]),color='gray',dashes=(2,3),clip_on=False,linewidth=2)
	ax.set_ylabel("Relative footprint density (AU)",fontdict=text_font)
	ax.set_xlabel("Normalized transcript length",fontdict=text_font,labelpad=30)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2)
	if xlabelLoc:
		ax.text(0,float(xlabelLoc),"5'UTR",fontdict=text_font)
		ax.text((int(bins_vector[0])+int(bins_vector[1])/3),float(xlabelLoc),"Coding region",fontdict=text_font)
		ax.text((int(bins_vector[0])+int(bins_vector[1])+int(bins_vector[2])/2),float(xlabelLoc),"3'UTR",fontdict=text_font)
	else:
		pass
	if not ymin and not ymax:
		pass
	elif not ymin and ymax:
		ax.set_ylim(0,ymax)
	elif ymin and not ymax:
		raise IOError("Please offer the ymax parameter as well!")
	elif ymin and ymax:
		ax.set_ylim(ymin,ymax)
	else:
		raise IOError("Please enter correct ymin and ymax parameters!")
	plt.legend(loc="best",prop=legend_font)
	# plt.legend(loc="best",prop={'size':10,'weight':'bold'})
	plt.tight_layout()
	plt.savefig(inOutPrefix+"_metaplot"+"."+inOutFomat,format=inOutFomat)
	plt.close()

def calculate_mean_data(data,samples,group_names,replicate_names,output_prefix):
	labels_dict={}
	for g,rep in zip(group_names,replicate_names):
		labels_dict[g]=rep.strip().split(',')
	if len(samples) < 1:
		raise IOError("There is no samples in the file, please check your input!")
	if len(samples) == 1:
		return data
	if len(samples) > 1:
		data_dict=defaultdict(list)
		data_mean_dict=defaultdict(list)
		for g in group_names:
			for r in labels_dict[g]:
				data_dict[g].append(data.loc[:,r])
		for g in group_names:
			if len(labels_dict[g]) <1:
				raise IOError("Please reset your -g -r parameters because nothing present here.")
			elif len(labels_dict[g]) ==1:
				data_mean_dict[g].append(list(reduce(zip,[data_dict[g][i].values for i in np.arange(len(data_dict[g]))])))
			elif len(labels_dict[g])==2:
				data_mean_dict[g].append(np.array([sum(list(chain(i))) for i in list(reduce(zip,[data_dict[g][i].values for i in np.arange(len(data_dict[g]))]))])/len(labels_dict[g]))
			else:
				data_mean_dict[g].append(np.array([reduce(sum,list(chain(i))) for i in list(reduce(zip,[data_dict[g][i].values for i in np.arange(len(data_dict[g]))]))])/len(labels_dict[g]))

		for k,v in data_mean_dict.items():
			data_mean_dict[k]=pd.DataFrame(v,index=[k]).T

		data_mean=pd.concat([v for v in data_mean_dict.values()],axis=1)
		## write the mean density file
		data_mean.to_csv(output_prefix+"_mean_scaled_dataframe.txt",sep="\t",index=None)
	return data_mean


def main():
	parsed=create_parser_for_the_whole_metagene_plot()
	(options,args)=parsed.parse_args()
	(data,output_prefix,group_names,replicate_names,output_format,ymin,ymax,bins,mode,xlabelLoc)=(options.density_file,
	options.output_prefix,options.group_name.strip().split(','),options.replicate_name.strip().split('__'),
	options.output_format,options.ymin,options.ymax,options.bins.strip(),options.mode,options.xlabelLoc)
	if len(bins.strip().split(',')) != 3:
		raise IOError("Please check your -b parameters!")
	print("your input file is: "+str(data),file=sys.stderr)
	data=pd.read_csv(data,sep="\t")
	samples=np.unique(data.columns)
	text_font={"size":20,"family":"Arial","weight":"bold"}
	legend_font={"size":20,"family":"Arial","weight":"bold"}
	text_font_mean={"size":20,"family":"Arial","weight":"bold"}
	legend_font_mean={"size":20,"family":"Arial","weight":"bold"}
	## calculate the mean density
	data_mean=calculate_mean_data(data,samples,group_names,replicate_names,output_prefix)
	samples_new=np.unique(data_mean.columns)
	if mode in ['all','All','a','A']:
		plot_read_coverage_distribution(data,samples,bins,output_prefix,output_format,ymin,ymax,xlabelLoc,text_font=text_font,legend_font=legend_font)
		plot_read_coverage_distribution(data_mean,samples_new,bins,output_prefix+"_mean",output_format,ymin,ymax,xlabelLoc,text_font=text_font,legend_font=legend_font)
		## plot density
		print("finished plot the ribosome footprint density",file=sys.stderr)
	elif mode in ['mean','Mean','m','M']:
		plot_read_coverage_distribution(data_mean,samples_new,bins,output_prefix+"_mean",output_format,ymin,ymax,xlabelLoc,text_font=text_font_mean,legend_font=legend_font_mean)
		print("finished plot the ribosome footprint density",file=sys.stderr)
	else:
		plot_read_coverage_distribution(data,samples,bins,output_prefix,output_format,ymin,ymax,xlabelLoc,text_font=text_font,legend_font=legend_font)
		plot_read_coverage_distribution(data_mean,samples_new,bins,output_prefix+"_mean",output_format,ymin,ymax,xlabelLoc,text_font=text_font_mean,legend_font=legend_font_mean)
		print("finished plot the ribosome footprint density",file=sys.stderr)


if __name__=="__main__":
	main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-23 15:14:11
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 17:09:01
@Description:

This script is used for getting ribosome density at specific region. For example, if there are ribosomes enriched on codon 25 to codon 75, we could
use this script to calculate the mean read density at 25-75 codons based on which we could split genes into different gene sets [up regulated or down regulated or unblocked].
The input is almost the same as MetageneAnalysis.py without filtering, so the output is the mean density of all longest transcripts.
'''


from .FunctionDefinition import *


def RibosomeDensity_for_specific_region(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,left_position,right_position,mode,unit):
		pysamFile=pysam.AlignmentFile(in_bamFile,'rb')
		pysamFile_trans=pysamFile.references
		in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
		local_mean_density={}
		local_density={}
		all_counts=0
		for trans in in_selectTrans:
				leftCoor =int(in_startCodonCoorDict[trans])-1
				rightCoor=int(in_stopCodonCoorDict[trans])-3
				(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
				all_counts+=total_reads
		for trans in in_selectTrans:
				leftCoor =int(in_startCodonCoorDict[trans])-1
				rightCoor=int(in_stopCodonCoorDict[trans])-3
				(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
				local_cds_counts=np.zeros(int(right_position-left_position+1),dtype="float64")
				local_cds_counts_normed=np.zeros(int(right_position-left_position+1),dtype="float64")
				## codon level
				if unit == 'codon':
						read_counts_frameSum_normed=10**6*(read_counts_frameSum/all_counts) ## RPM
						temp_local_cds_counts=read_counts_frameSum[int(left_position-1):int(right_position)] ## could offer two parameters later on
						temp_local_cds_counts_normed=read_counts_frameSum_normed[int(left_position-1):int(right_position)]
						local_cds_counts[np.arange(len(temp_local_cds_counts))]=temp_local_cds_counts
						local_cds_counts_normed[np.arange(len(temp_local_cds_counts_normed))]=temp_local_cds_counts_normed
						if mode == 'RPKM':
								if np.sum(local_cds_counts_normed) == 0:
										tmp_local_cds_counts_normed=0
										local_mean_density[trans]=tmp_local_cds_counts_normed
								else:
										local_mean_density[trans]=np.mean(local_cds_counts_normed)
								local_density[trans]=local_cds_counts_normed
						if mode == 'counts':
								if np.sum(local_cds_counts) == 0:
										tmp_local_cds_counts=0
										local_mean_density[trans]=tmp_local_cds_counts
								else:
										local_mean_density[trans]=np.mean(local_cds_counts)
								local_density[trans]=local_cds_counts
				if unit == 'nt':
						# trans_counts_normed=10**9*(trans_counts/all_counts*len(trans_counts))
						trans_counts_normed=10**6*(trans_counts/all_counts)
						temp_local_cds_counts=trans_counts[int(left_position-1):int(right_position)] ## could offer two parameters later on
						temp_local_cds_counts_normed=trans_counts_normed[int(left_position-1):int(right_position)]
						local_cds_counts[np.arange(len(temp_local_cds_counts))]=temp_local_cds_counts
						local_cds_counts_normed[np.arange(len(temp_local_cds_counts_normed))]=temp_local_cds_counts_normed
						if mode == 'RPKM':
								if np.sum(local_cds_counts_normed) == 0:
										tmp_local_cds_counts_normed=0
										local_mean_density[trans]=tmp_local_cds_counts_normed
								else:
										local_mean_density[trans]=np.mean(local_cds_counts_normed)
								local_density[trans]=local_cds_counts_normed
						if mode == 'counts':
								if np.sum(local_cds_counts) == 0:
										tmp_local_cds_counts=0
										local_mean_density[trans]=tmp_local_cds_counts
								else:
										local_mean_density[trans]=np.mean(local_cds_counts)
								local_density[trans]=local_cds_counts

		return local_mean_density,local_density

def write_local_codon_units_density(inBamAttr,outFile,left_position,right_position):
	for bms in inBamAttr:
		data=pd.DataFrame(bms.local_density)
		data=data.T
		data.columns=["codon_"+str(i+1) for i in np.arange(int(right_position)+1-int(left_position))]
		data.to_csv(outFile+"_"+bms.bamLegend+"_local_density.txt",sep='\t')


def write_bam_file_local_mean_cds_counts_dataframe(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		d=bms.local_mean_density
		i=bms.bamLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")

def parse_args_for_specific_region_metagene():
	parsed=creat_parser_for_specific_region()
	(options,args)=parsed.parse_args()
	if options.bamListFile and (options.bam_files or options.read_length or options.read_offset or options.bam_file_legend):
		raise IOError("'-f' parameter and '-i -r -s -t' are mutually exclusive.")
	if options.bamListFile:
		bamFiles,readLengths,Offsets,bamLegends=parse_bamListFile(options.bamListFile)
	elif options.bam_files:
		bamFiles,readLengths,Offsets,bamLegends=options.bam_files.split(","),options.read_length.split("_"),options.read_offset.split("_"),options.bam_file_legend.split(",")
	else:
		raise IOError("Please check you input files!")
	print("your input : "+ str(len(bamFiles))+" bam files",file=sys.stderr)
	bam_attr=[]
	for ii,jj,mm,nn in zip(bamFiles,readLengths,Offsets,bamLegends):
		bam=bam_file_attr(ii,jj,mm,nn)
		bam_attr.append(bam)
	## calculate density for each bam files
	selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict=reload_transcripts_information(options.coorFile)
	geneID2transID={v:k for k,v in transID2geneID.items()}
	geneName2transID={v:k for k,v in transID2geneName.items()}
	if options.in_selectTrans:
		select_trans=pd.read_csv(options.in_selectTrans,sep="\t")
		select_trans=set(select_trans.iloc[:,0].values)
		if options.id_type == 'transcript_id':
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans)) + " transcripts from "+options.in_selectTrans+" used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_id':
			tmp=[geneID2transID[gene_id] for gene_id in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene id could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		elif options.id_type == 'gene_name' or options.id_type=='gene_symbol':
			tmp=[geneName2transID[gene_name] for gene_name in select_trans]
			select_trans=set(tmp)
			select_trans=select_trans.intersection(selectTrans)
			print("There are " + str(len(select_trans))+" gene symbol could be transformed into transcript id and used for following analysis.",file=sys.stderr)
		else:
			raise IOError("Please input a approproate id_type parameters.[transcript_id/gene_id/gene_name/]")
	else:
		select_trans=selectTrans

	print("There are "+str(len(select_trans))+" transcripts with both start and stop codon will be used for following analysis.",file=sys.stderr)
	for bamfs in bam_attr:
			(bamfs.local_mean_density,bamfs.local_density)=RibosomeDensity_for_specific_region(bamfs.bamName,select_trans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,
			bamfs.bamLen,bamfs.bamOffset,options.left_position,options.right_position,options.mode,options.unit)
	print("Finish the step of ribosomeDensityNormPerTrans",file=sys.stderr)
	## write density
	write_bam_file_local_mean_cds_counts_dataframe(bam_attr,options.output_prefix+"_local_mean_density.txt")
	write_local_codon_units_density(bam_attr,options.output_prefix,options.left_position,options.right_position)
	print("Finish the step of write_bam_file_density",file=sys.stderr)

def main():
		'''main funciton'''
		parse_args_for_specific_region_metagene()

if __name__=="__main__":
		main()



#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-12 17:52:34
@LastEditors: Li Fajin
@LastEditTime: 2019-09-30 15:37:47
@Description:
	Containing some common functions used for other scripts:
	1. bam_file_attr(): a class used for define attribution of bam files.
	2. parse_gtfFile(): a function used for parsing gtf file.
	3. parse_coorFile(): a function used for parsing transcript_cds.txt file generated by RiboCode.
	4. get_trans_length_dict(): a function used for getting a length dict of all transcripts based on transcripts_sequence.fa generated by RiboCode.
	5. get_longest_transcripts_information(): a function used for getting longest transcript information of protein coding genes.
	6. get_all_transcripts_information(): a function used for getting all transcript information.
	7. reload_transcripts_information(): a function used for re-get longest transcript information based on file generated by get_longest_transcripts_information().
	8. getWindowsVector(): a function used for lining up all transcripts density vectors when do the metagene analysis.
	9. lengths_offsets_split(): a function used for getting read length and offset separated by comma.
	10. get_trans_frame_counts(): a function used for counts calculation at each position along transcripts.
	11. RPKM_of_all_genes(): a function used for RPKM calculation based on reads covered on the whole transcripts or on CDS region.
	12. write_bam_file_read_counts_dataframe(): function used for output RPKM values.
	13. write_bam_file_density_dataframe(): a function used for outputing density dataframe if not set --CI parameter.
	14. parse_bamListFile(): a function used for parsing bam attribute files constructured by user themselves.
	15. fasta_attrbution(): a class used for define attribution of fasta files.
	16. fastaIter(): a function used for getting a sequence dict based on a fasta file.
	17. translation(): a function used for translating a DNA seq to AA seq.
	18. flatten(): a function used for flatten all iterable objects.
	19. create_parser_for_cAI()
	20. create_parser_for_enrichment_analysis()
	21. create_parser_for_single_ratio_plot()
	22. create_parser_for_sequence_extraction()
	23. create_parser_for_UTR_sequence_extraction()
	24. create_parser_for_hydropathy_or_charge()
	25. create_parser_for_metagene_analysis()
	26. create_parser_for_metagene_analysis_for_the_whole_regions()
	27. create_parser_for_output_transInfo()
	28. create_parse_for_pausing_score_calculation()
	29. create_parser_for_periodicity()
	30. create_parser_for_polarity_calculation()
	31. create_parser_for_triplete_AA_motif()
	32. creat_parser_for_calculation_of_codon_density()
	33. create_parser_for_riboDensity_atEachPosition()
	34. creat_parser_for_specific_region()
	35. create_parser_for_frame_density()
	36. create_parser_for_RPFdist()
	37. create_parser_for_tAI()
'''
from __future__ import division
from __future__ import absolute_import
from __future__ import print_function
from .__init__ import __version__
import sys
import os
import pysam
import itertools
from itertools import groupby
import numpy as np
import pandas as pd
from Bio.Seq import translate
from optparse import OptionParser


class bam_file_attr(object):
	"""Class for bam file attribute"""
	def __init__(self,bamName,bamLen,bamOffset,bamLegend):
		self.bamName=bamName
		self.bamLen=bamLen
		self.bamOffset=bamOffset
		self.bamLegend=bamLegend

class fasta_attrbution(object):
	"""Class for fasta file attribute"""
	def __init__(self,fastaName,fastaLegend):
		self.fastaName=fastaName
		self.fastaLegend=fastaLegend

def parse_gtfFile(gtfFile):
		'''extract trancript information from standard GTF file'''
		transID2GeneIDDict={}
		geneIDNameDict={}
		proteinCodingGeneDict={}
		transWithStrand={}
		transID_start_coor={}
		transID_stop_coor={}
		geneID_Biotype_Dict={}
		with open(gtfFile,'r') as f:
			for line in f:
				if line.strip()[0] == "#" or (not line.strip()):
					continue
				fields=line.strip().split("\t")
				strand=fields[6]
				trans_start=str(fields[3])
				trans_stop=str(fields[4])
				if len(fields) < 9:
					raise KeyError("Sorry, you get a unregular GTF file. Please check it again.")
				KeyDesc={i.strip().split(" ")[0]:i.strip().split(" ")[1].strip('"') for i in fields[8].strip(';').split(';')}
				if fields[2]=='transcript' and ('transcript_biotype "protein_coding"' in line.strip() or 'transcript_type "protein_coding"' in line.strip()) :
					geneID=KeyDesc['gene_id']
					transID=KeyDesc['transcript_id']
					transID2GeneIDDict[transID]=geneID
					transWithStrand[transID]=strand
					transID_start_coor[transID]=trans_start
					transID_stop_coor[transID]=trans_stop
					if "gene_name" not in KeyDesc.keys():
						geneName=geneID
						biotype=KeyDesc['transcript_biotype']
					else:
						geneName=KeyDesc['gene_name']
						biotype=KeyDesc['transcript_biotype']
					geneIDNameDict[geneID]=geneName
					geneID_Biotype_Dict[geneID]=biotype
					if geneID not in proteinCodingGeneDict:
						proteinCodingGeneDict[geneID]=set([transID])
					else:
						proteinCodingGeneDict[geneID].add(transID)
		return transID2GeneIDDict,geneIDNameDict,proteinCodingGeneDict,transWithStrand,transID_start_coor,transID_stop_coor,geneID_Biotype_Dict

def parse_coorFile(coorFile):
		'''extract transcript information from coorFile generated by RiboCode'''
		startCodonCoorDict={}
		stopCodonCoorDict={}
		with open(coorFile,'r') as f:
			for coors in f:
				startCodonCoorDict[coors.strip().split('\t')[0]]=coors.strip().split('\t')[1]
				stopCodonCoorDict[coors.strip().split('\t')[0]]=coors.strip().split('\t')[2]
		return startCodonCoorDict,stopCodonCoorDict

def get_trans_length_dict(transcriptFile):
	'''
	This function is used to get a dict of transcript length
	'''
	trans_length_dict={}
	transFile=open(transcriptFile,'r')
	faiter=(x[1] for x in groupby(transFile,lambda line: line.strip()[0]==">")) ## groupby returns a tuple (key, group)
	for header in faiter:
		trans_id=header.__next__().strip(">").split(" ")[0]
		seq=''.join(s.strip() for s in faiter.__next__())
		length=int(len(seq.strip()))
		trans_length_dict[trans_id]=length
	return trans_length_dict

def get_longest_transcripts_information(coorFile,transcriptFile,gtfFile,longestTransFile):
	'''
	This function is aimed to get four kinds of information:
	1) all transcripts id of the longest isoform of all protein coding genes containing both start codon and stop codon: selectTrans
	2) start codon dictionary of all transcripts selected above: startCodonCoorDict
	3) stop codon dictionary of all transcripts selected above: stopCodonCoorDict
	4) transcript length dictionary of all transcripts selected above: transLengthDict
	'''
	transID2GeneIDDict,geneIDNameDict,proteinCodingGeneDict,transWithStrand,transID_start_coor,transID_stop_coor,geneID_Biotype_Dict=parse_gtfFile(gtfFile)
	startCodonCoorDict,stopCodonCoorDict=parse_coorFile(coorFile)
	## select the longest transcript for each gene
	transLengthDict=get_trans_length_dict(transcriptFile)
	selectLongestTrans=set()
	for trans_id in proteinCodingGeneDict.values():
		if len(trans_id) >1:
			tmp = sorted(list(trans_id),key=lambda x: transLengthDict[x])[-1]
			selectLongestTrans.add(tmp)
		else:
			selectLongestTrans.add(list(trans_id)[0])
	## select transcripts with both start codon and stop codon
	transWithBothStartAndStopCodon=set(startCodonCoorDict.keys()).intersection(set(stopCodonCoorDict.keys()))
	selectTrans=transWithBothStartAndStopCodon.intersection(set(transLengthDict.keys()))
	selectTrans=selectTrans.intersection(selectLongestTrans)
	print(str(len(selectTrans))+'  transcripts will be used in the follow analysis.\n', file=sys.stderr)
	## output the longest transcript of each gene
	with open(longestTransFile,'w') as fout:
		fout.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" %("trans_id","strand","gene_id","gene_name","gene_biotype","gene_start","gene_stop","trans_start","trans_start","CDS_length","5UTR_length","3UTR_length","transcript_length"))
		for ltid in selectLongestTrans:
			geneID=transID2GeneIDDict[ltid]
			cds_length=int(stopCodonCoorDict[ltid])-int(startCodonCoorDict[ltid])+1
			Five_UTR_length=int(startCodonCoorDict[ltid])-1
			Three_UTR_length=int(transLengthDict[ltid])-int(stopCodonCoorDict[ltid])
			fout.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (ltid,transWithStrand[ltid],geneID,geneIDNameDict[geneID],geneID_Biotype_Dict[geneID],transID_start_coor[ltid],transID_stop_coor[ltid],startCodonCoorDict[ltid],stopCodonCoorDict[ltid],cds_length,str(Five_UTR_length),str(Three_UTR_length),str(transLengthDict[ltid])))

def get_all_transcripts_information(coorFile,transcriptFile,gtfFile,allTranscriptsInfo):
	with open(gtfFile) as fin:
		transID_geneID_Dict={}
		geneID_Name_Dict={}
		geneID_Biotype_Dict={}
		transID_strand_Dict={}
		transID_start_coor={}
		transID_stop_coor={}
		for line in fin:
			if line[0] == "#" or (not line.strip()):
				continue
			fields=line.strip().split("\t")
			strand=strand=fields[6]
			trans_start=str(fields[3])
			trans_stop=str(fields[4])
			if len(fields) < 9:
					raise KeyError("Sorry, you get a unregular GTF file. Please check it again.")
			KeyDesc={i.strip().split(" ")[0]:i.strip().split(" ")[1].strip('"') for i in fields[8].strip(';').split(';')}
			if line.strip().split('\t')[2]=='transcript' :
				transID=KeyDesc['transcript_id']
				geneID=KeyDesc['gene_id']
				transID_geneID_Dict[transID]=geneID
				transID_strand_Dict[transID]=strand
				transID_start_coor[transID]=trans_start
				transID_stop_coor[transID]=trans_stop
				if "gene_name" not in KeyDesc.keys():
					#some genes dont have gene_name but everyone has the gene_id
					# print(geneID)
					geneName=geneID
					biotype=KeyDesc['transcript_biotype']
				else :
					geneName=KeyDesc['gene_name']
					biotype=KeyDesc['transcript_biotype']
				geneID_Name_Dict[geneID]=geneName
				geneID_Biotype_Dict[geneID]=biotype
	###get start_codon stop_codon transcript coordinate
	startCodonCoorDict,stopCodonCoorDict=parse_coorFile(coorFile)
	## get transcript length
	transLengthDict=get_trans_length_dict(transcriptFile)
	#####write Dict to outfile
	with open(allTranscriptsInfo,"w") as fout:
		fout.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % ("trans_id","strand","gene_id","gene_name","gene_biotype","gene_start","gene_stop","trans_start","trans_stop","CDS_length","5UTR_length","3UTR_length","transcript_length"))
		for fn in sorted( startCodonCoorDict.keys() ):
			geneID=transID_geneID_Dict[fn]
			cds=int(stopCodonCoorDict[fn])-int(startCodonCoorDict[fn])+1
			Five_UTR=int(startCodonCoorDict[fn])-1
			Three_UTR=int(transLengthDict[fn])-int(stopCodonCoorDict[fn]) ## length-stop+1-3
			fout.write("%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\n" % (fn,transID_strand_Dict[fn],geneID,geneID_Name_Dict[geneID],geneID_Biotype_Dict[geneID],str(transID_start_coor[fn]),str(transID_stop_coor[fn]),str(startCodonCoorDict[fn]),str(stopCodonCoorDict[fn]),str(cds),str(Five_UTR),str(Three_UTR),str(transLengthDict[fn])))


def reload_transcripts_information(longestTransFile):
	selectTrans=set()
	transLengthDict={}
	cdsLengthDict={}
	startCodonCoorDict={}
	stopCodonCoorDict={}
	transID2geneID={}
	transID2geneName={}
	with open(longestTransFile,'r') as f:
		for line in f:
			if line.strip()=='':
				continue
			if line.strip().split("\t")[0] == 'trans_id':
				continue
			transID=line.strip().split("\t")[0]
			geneID=line.strip().split("\t")[2]
			geneName=line.strip().split("\t")[3]
			startCodon=int(line.strip().split("\t")[7])
			stopCodon=int(line.strip().split("\t")[8])
			cds_length=int(line.strip().split("\t")[9])
			transLength=int(line.strip().split("\t")[12])
			selectTrans.add(transID)
			transLengthDict[transID]=transLength
			startCodonCoorDict[transID]=startCodon
			stopCodonCoorDict[transID]=stopCodon
			transID2geneID[transID]=geneID
			transID2geneName[transID]=geneName
			cdsLengthDict[transID]=cds_length
			# print(transID,geneID,geneName,startCodon,stopCodon,transLength)
	print(str(len(selectTrans))+'  transcripts will be used in the follow analysis.\n', file=sys.stderr)
	return selectTrans,transLengthDict,startCodonCoorDict,stopCodonCoorDict,transID2geneID,transID2geneName,cdsLengthDict



def getWindowsVector(upLength,downLength,transVector,inCoor):
		"""
		get every transcript windows reads density vector
		"""
		## inCoor is 0-based coordinate
		windowsVector=np.zeros(int(upLength+downLength+1),dtype="float64")
		posVector=np.zeros(int(upLength+downLength+1),dtype="int64")

		## left
		if inCoor <= upLength :
				transLeftIndex=0
				windowsLeftIndex=upLength-inCoor
		else:
				transLeftIndex=inCoor-upLength
				windowsLeftIndex=0

		## right
		if len(transVector)-(inCoor+1) <= downLength:
				transRightIndex=len(transVector) ## python splice do not contain the last one position,so the last index should be len(transVector)-1+1
				windowsRightIndex=upLength-inCoor+len(transVector)
		else:
				transRightIndex=inCoor+1+downLength
				windowsRightIndex=len(windowsVector)
		## final vector for that transcript
		windowsVector[windowsLeftIndex:windowsRightIndex]+=transVector[transLeftIndex:transRightIndex]
		posVector[windowsLeftIndex:windowsRightIndex]+=1
		return (windowsVector,posVector)
def lengths_offsets_split(value):
		''' Split the given comma separated values to multiple integer values'''
		values=[]
		for item in value.split(','):
				item=int(item)
				values.append(item)
		return values

def get_trans_frame_counts(ribo_fileobj, transcript_name, read_lengths, read_offsets, transLength, startCoor, stopCoor):
	"""For each mapped read of the given transcript in the BAM file,get the P-site and codon unit reads density
	ribo_fileobj -- file object - BAM file opened using pysam AlignmentFile
	transcript_name -- Name of transcript to get counts for
	read_length -- If provided, get counts only for reads of this length.
	read_offsets -- the offset length corresponding to 5' mapped position.
	transLength -- the length of the transcript.
	startCoor -- the coordinate of the first base of start codon 0-based.
	stopCoor -- the coordinate of the first base of stop codon 0-based.
	"""
	read_counts = np.zeros(transLength,dtype="int64")
	total_reads = 0
	if read_lengths == "ALL" : ## RNA
		for record in ribo_fileobj.fetch(transcript_name):
			if record.flag == 16 or record.flag == 272:
				continue
			total_reads += 1
			position = record.pos
			read_counts[position]+=1
	else:
		read_lengths=lengths_offsets_split(read_lengths)
		read_offsets=lengths_offsets_split(read_offsets)
		for record in ribo_fileobj.fetch(transcript_name):
			if record.flag == 16 or record.flag == 272:
				continue
			for R_length, R_offset in zip(read_lengths,read_offsets):
				if  record.query_length == R_length :
					# if an offset is specified, increment position by that offset.
					position = record.pos + R_offset ## transform into the position of P-site
				else:
					# ignore other reads/lengths
					continue
				total_reads += 1
				try:
					read_counts[position]+=1
				except KeyError:
					print("Dont has this position after offset : transcript_name -> position"+" "+transcript_name+" -> "+position)
	#get trans counts for each 3 frames
	read_counts_frame0=read_counts[(startCoor+0):(stopCoor-2):3]
	read_counts_frame1=read_counts[(startCoor+1):(stopCoor-1):3]
	read_counts_frame2=read_counts[(startCoor+2):(stopCoor-0):3]
	read_counts_frameSum=read_counts_frame0+read_counts_frame1+read_counts_frame2
	cds_reads=sum(read_counts_frameSum)
	return read_counts,read_counts_frameSum,total_reads,cds_reads


def RPKM_of_all_genes(in_bamFile,in_selectTrans,in_transLengthDict,in_startCodonCoorDict,in_stopCodonCoorDict,in_readLengths,in_readOffset,Type):
	'''calculate the RPKM values for the CDS region or all transcript region'''
	pysamFile=pysam.AlignmentFile(in_bamFile,'rb')
	pysamFile_trans=pysamFile.references
	in_selectTrans=set(pysamFile_trans).intersection(in_selectTrans)
	# in_selectTrans=list(in_selectTrans)
	RPKM={}
	all_counts=0
	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		if Type.upper() in ['CDS']:
			all_counts+=cds_reads
		elif Type.upper() in ['TRANSCRIPT','TRANS','TRANSCRIPTS']:
			all_counts+=total_reads
		else:
			raise IOError("please choose your type of RPKM.[CDS or exon]")
	for trans in in_selectTrans:
		leftCoor =int(in_startCodonCoorDict[trans])-1
		rightCoor=int(in_stopCodonCoorDict[trans])-3
		(trans_counts,read_counts_frameSum,total_reads,cds_reads)=get_trans_frame_counts(pysamFile, trans, in_readLengths, in_readOffset, in_transLengthDict[trans], leftCoor, rightCoor)
		if Type.upper() in ['CDS']:
			cds_reads_normed=10**9*(cds_reads/(all_counts*len(read_counts_frameSum)))
			RPKM[trans]=cds_reads_normed
		elif Type.upper() in ['TRANSCRIPT','TRANS','TRANSCRIPTS','EXON','EXONS']:
			trans_counts_normed=10**9*(total_reads/(all_counts*len(trans_counts)))
			RPKM[trans]=trans_counts_normed
	return RPKM

def write_bam_file_density_dataframe(inBamAttr,outFile):
	data=[]
	for bms in inBamAttr:
		k=pd.DataFrame([bms.bamLegend]*len(bms.start_density))
		start=pd.DataFrame(bms.start_density)
		stop=pd.DataFrame(bms.stop_density)
		density=pd.merge(start,stop,how="left",left_index=True,right_index=True)
		density=pd.merge(k,density,how="left",left_index=True,right_index=True)
		data.append(density)
	temp=data[0]
	if len(data) < 1:
		raise EOFError("Empty file, there is nothing in the file.")
	if len(data) == 1:
		temp.columns=['sample','start_density','stop_density']
		temp.to_csv(outFile,sep="\t",index=0)
	else:
		for i in np.arange(1,len(data)):
			temp=np.vstack((temp,data[i]))
		temp=pd.DataFrame(temp,columns=["sample","start_density","stop_density"])
		temp.to_csv(outFile,sep="\t",index=0)

def write_bam_file_read_counts_dataframe(inBamAttr,outFile):
	data=[]
	data_index=[]
	for bms in inBamAttr:
		d=bms.RPKM
		i=bms.bamLegend
		data.append(d)
		data_index.append(i)
	data=pd.DataFrame(data,index=data_index)
	data=data.T
	data.to_csv(outFile,sep="\t")

def parse_bamListFile(bamListFile):
	bamFileList=[]
	readLengthsList=[]
	OffsetsList=[]
	bamLegendsList=[]
	flag=1
	with open(bamListFile,'r') as f:
		for line in f:
			if flag == 1:
				flag+=1
				continue
			bamFile=line.strip().split("\t")[0]
			readLengths=line.strip().split("\t")[1]
			Offsets=line.strip().split("\t")[2]
			bamLegends=line.strip().split("\t")[3]
			bamFileList.append(bamFile)
			readLengthsList.append(readLengths)
			OffsetsList.append(Offsets)
			bamLegendsList.append(bamLegends)
	return bamFileList,readLengthsList,OffsetsList,bamLegendsList

def fastaIter(transcriptFile):
	'''
	This function is used to get a dict of transcript sequence
	'''
	fastaDict={}
	f=open(transcriptFile,'r')
	faiter=(x[1] for x in groupby(f,lambda line: line.strip()[0]==">")) ## groupby returns a tuple (key, group)
	for header in faiter:
		geneName=header.__next__().strip(">").split(" ")[0]
		seq=''.join(s.strip() for s in faiter.__next__())
		fastaDict[geneName]=seq
	return fastaDict

def translation(seq,table=1,cds=True):
	"""
	translate the DNA to protein sequence using the translation table
	table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi
	Code from RiboCode.[Xiao,et al. NAR. 2018]
	"""
	if len(seq) % 3 != 0:
		sys.stderr.write("Warning: sequence is not divisible by 3\n")
		seq = seq[:-(len(seq) % 3)]
	return translate(seq,table=table,cds=cds)

def flatten(xs):
	for x in xs:
		if isinstance(x,tuple):
			for xx in flatten(x):
				yield xx
		else:
			yield x

## create parser for different scripts
def create_parser_for_cAI():
	'''argument parser.'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="transcriptFiles",
			help="Input file(s) in fasta format. All files should be split by comma e.g. 1.fasta,2.fasta,3.fasta[required]\n. Note: input sequence must be cds sequences which could be generated by GetProteinCodingSequence.py.")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-t","--trans_file_legend",action="store",type="string",dest="trans_file_legend",
			help="The legend of each fasta file.[required]")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",
			help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.default=%default")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon",
			help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.default=%default")
	parser.add_option('--reference',action='store',type='string',default=None,dest='reference',
			help="A reference cds sequences used for calculating the relative synonymous codon usage [RSCU].")
	parser.add_option("--RSCU",action="store",type="string",default=None,dest="RSCUs",
			help="A RSCU files used for calculating each codon weight. default=%default.")
	parser.add_option("--weight",action="store",type="string",default=None,dest="weights",
			help="A weight file used for calculating the cAI.")
	parser.add_option("--table",action="store",type="int",default=1,dest="genetic_table",
			help="The genetic code to used. default=1. table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	return parser

def create_parser_for_enrichment_analysis():
	'''argument parser.'''
	usage="usage: %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("--ctrl", action="store",type="string",default=None,dest="ctrlDensity",
			help="Control density file generated by RiboDensityAtEachPosition.py.[required].")
	parser.add_option("--treat", action="store",type="string",default=None,dest="treatDensity",
			help="Treat density file generated by RiboDensityAtEachPosition.py.[required].")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-U","--unit_type",action="store",type="string",dest="unit", default="codon",
			help="Unit type we used for metagene analysis. Either 'nt' or 'codon'. default=%default")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=128,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-m","--minimum_norm_region_counts",action="store",type="int",default=64,dest="min_norm_region_counts",
			help="Minimum counts in normalization region . Counts in norm region smaller than \"minimum_norm_region_counts\" will be skipped. default=%default")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",
			help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon",
			help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-e","--normalize_exclude_codon",action="store",type="int",default=30, dest="norm_exclude_codon",
			help="The region of the normalization for each genes should exclude the first \"normalize_exclude_codon\". default=%default")
	parser.add_option("--CI",action="store",type='float',dest="confidence",default=0.95,
			help="The confidence intervals of mean on each position based on t distribution. default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	return parser

def create_parser_for_single_ratio_plot():
	'''argument parser.'''
	usage="usage: python %prog [options]" + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="ratioFile",
			help="All ratio file generated by EnrichmentAnalysis.py.")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option('-s','--single-trans',action="store",type='string',dest='singleTrans',
			help="A single selected transcript.")
	parser.add_option('-S','--selected-trans-list',action="store",type="string",dest="in_selectTrans",
			help="Selected transcripts list. The first column of this file must be transcript id or gene identifier with a column name.")
	parser.add_option("--ymin",action="store",type="float",dest="ymin",default=None,help="The max of ylim. default=%default")
	parser.add_option("--ymax",action="store",type="float",dest="ymax",default=None,help="The max of ylim. default=%default")
	parser.add_option("--unit",action="store",type="string",dest="unit",default='codon',help="Unit for density calculation.[codon or nt]")
	parser.add_option("--axvline",action="store",type="float",dest="axvline",default=None,help="Position to plot a vertical line in x axis. default=%default")
	parser.add_option("--axhline",action="store",type="float",dest="axhline",default=None,help="Position to plot a vertical line in y axis. default=%default")
	parser.add_option("--slide-window",action="store",type="string",dest="slideWindow",default=None,help="Using slide window to average the density.Input a	true strings such as yes, y or 1. %default=default")
	parser.add_option("--start",action="store",type="int",dest="start_position",default=5,help="The start position need to be averaged.default=%default")
	parser.add_option("--window",action="store",type="int",dest="window",default=7,help="The length of silde window. default=%default")
	parser.add_option("--step",action="store",type='int',dest="step",default=1,help="The step length of slide window. default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. %default=default")
	return parser

def create_parser_for_sequence_extraction():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="transcriptFile",
			help="Input file(s) in fasta format. generated by RiboCode")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-l","--left_coor",action="store",type="int",dest="left_position",
			help="The left position (codon level) of the sequence you  wanna to extract.[required]")
	parser.add_option("-r","--right_coor",action="store",type="int",dest="right_position",
			help=" The right position (codon level) of the sequence you  wanna to extract[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("--mode",action="store",type="string",dest="mode",default="whole",
			help="Mode for sequence extraction.[whole or local]")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	return parser

def create_parser_for_UTR_sequence_extraction():
	'''argument parser'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="transcriptFile",
			help="Input file(s) in fasta format. All transcript sequences are from the longest transcripts of protein coding genes which may be generated by GetProteinCodingSequence.py")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by RiboCode.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	return parser

def create_parser_for_hydropathy_or_charge():
	'''argument parser.'''
	usage="usage: %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="transcriptFiles",
			help="Input file(s) in fasta format. All files should be split by comma e.g. 1.fasta,2.fasta,3.fasta[required]\n. Note: input sequence must be cds sequences which could be generated by GetProteinCodingSequence.py.")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-t","--trans_file_legend",action="store",type="string",dest="trans_file_legend",
			help="The legend of each fasta files, comma split. e.g. 'condition1,condition2,condition3' [required]")
	parser.add_option("--index",action='store',type='string',dest='index',
			help="Input the hydropathy index or charge index of each amino acids.")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",
			help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon",
			help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	return parser

def create_parser_for_metagene_analysis():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-U","--unit_type",action="store",type="string",dest="unit", default="codon",
			help="Unit type we used for metagene analysis. Either 'nt' or 'codon'. default=%default")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=128,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-m","--minimum_norm_region_counts",action="store",type="int",default=64,dest="min_norm_region_counts",
			help="Minimum counts in normalization region . Counts in norm region smaller than \"minimum_norm_region_counts\" will be skipped. default=%default")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",
			help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon",
			help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.")
	parser.add_option("-e","--normalize_exclude_codon",action="store",type="int",default=30, dest="norm_exclude_codon",
			help="The region of the normalization for each genes should exclude the first \"normalize_exclude_codon\". default=%default")
	parser.add_option("-y","--normalize_codon_density_num",action="store",type="int",default=100, dest="norm_codon_density_num",
			help="Output the first \"normalize_codon_density_num\" from the start codon for each qualified genes in each bam files. Pay attention that this parameter should not bigger than -l parameter. default=%default")
	parser.add_option("--type",action='store',type='string',dest='type',default='CDS',help='Type of counts statistics.[CDS/cds or UTR/utr].default=%default')
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	parser.add_option("--CI",action="store",type='float',dest="confidence",default=0.95,
			help="The confidence intervals of mean on each position based on t distribution. default=%default")
	parser.add_option("--norm",action="store",type="string",dest="norm",default='yes',
			help="Use density beyound the first 30 (-e) codons for normalization or not. [yes/no]")
	parser.add_option("--plot",action="store",type="string",dest="plot",default='yes',help="Output a metagene plot or not. default=%default")
	return parser

def create_parser_for_metagene_analysis_for_the_whole_regions():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-b","--bins",action='store',type='string',dest="bins",default='15,90,60',
			help="Bins to scale the transcript length.e.g.'15,90,60'. bins must be separated by comma, namely '5UTRBins,CDSBins,3UTRBins'. default=%default")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=128,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-m","--minimum_norm_region_counts",action="store",type="int",default=64,dest="min_norm_region_counts",
			help="Minimum counts in normalization region . Counts in norm region smaller than \"minimum_norm_region_counts\" will be skipped. default=%default")
	parser.add_option("-e","--normalize_exclude_codon",action="store",type="int",default=30, dest="norm_exclude_codon",
			help="The region of the normalization for each genes should exclude the first \"normalize_exclude_codon\". default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. %default=default")
	parser.add_option("--plot",action="store",type="string",dest="plot",default='yes',
			help="Output a metagene plot or not.default=%default")
	return parser

def create_parser_for_output_transInfo():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon of each transcript. Generated by RiboCode.[required]")
	parser.add_option("-g","--gtfFile",action="store",type="string",dest="gtfFile",
			help="Reference gene annotation in gtf format.[required]")
	parser.add_option("-f","--transcriptFile",action="store",type="string",dest="transcriptFile",
			help="transcript file with fasta format generated by RiboCode.[required]")
	parser.add_option("-o","--output-longest-transcrpts",action="store",type="string",dest="longestTransInfo",
			help="Longest transcripts information.")
	parser.add_option("-O","--output-all-transcrpts",action="store",type="string",dest="allTransInfo",
			help="All transcripts information.")
	return parser

def create_parse_for_pausing_score_calculation():
	'''arguments definition'''
	usage="usage: python %prog [options]" + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")

	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=64,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-L","--Left_position",action="store",type="int",dest="left_position", default=None,
			help="Left position of the interesting region [codon level]. default=%default")
	parser.add_option("-R","--Right_position",action="store",type="int",dest="right_position", default=None,
			help="Right position of the interesting region [codon level]. default=%default")
	parser.add_option("-F","--transcript_fasta",action="store",type="string",dest="transcript_fasta",
			help="Input the transcript fasta file used for motif detection and codon density calculation. cds sequences related with coorFile (longest.trans.info.txt) generated by GetProteinSequence.py")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")

	return parser

def create_parser_for_periodicity():
	'''argument parser.'''
	usage="usage: python %prog [options]" + "\n"
	parser=OptionParser(usage=usage)
	parser.add_option("-i","--input", action="store",type="string",dest="bamFile",
			help="Input file(s) in bam format.")
	parser.add_option("-a","--annot_dir",action="store",type="string",dest="annot_dir",
	                    help="transcripts annotation directory, generated by prepare_transcripts, a function of RiboCode.")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-L","--left_length",action="store",type="int",default=25,dest="left_length",help="The left range of read length we will consider.")
	parser.add_option("-R","--right_length",action="store",type="int",default=35,dest="right_length",help="The right range of read length we will consider.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. %default=default")
	return parser

def create_parser_for_polarity_calculation():
	'''argument parser.'''
	usage='usage: python %prog[options]'
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
		help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=64,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
		help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. %default=default")
	parser.add_option("--plot",action="store",type="string",dest="plot",default='yes',
			help="Output a polarity plot or not.default=%default")
	return parser

def create_parser_for_triplete_AA_motif():
	'''argument parser.'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option('--type1',action='store',type='string',dest='type1',default=None,
			help="The first motif you want to find. It must be a string with length equal to 2. e.g. PP,DD,KK motifs. default=%default.")
	parser.add_option('--type2',action='store',type='string',dest='type2',default=None,
			help="The second motif you want to find. It must be a string with length equal to 3. e.g. PPP,DDD,KKK motifs. default=%default.")
	parser.add_option('--motifList1',action='store',type='string',dest='motifList1',default=None,
			help="The first motif list you want to find. Mutually exclusive with '--type1' '--type2'. default=%default")
	parser.add_option('--motifList2',action='store',type='string',dest='motifList2',default=None,
			help="The second motif list you want to find. Mutually exclusive with '--type1' '--type2'. default=%default")
	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=64,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-F","--transcript_fasta",action="store",type="string",dest="transcript_fasta",
			help="Input the transcript fasta file used for motif detection and codon density calculation. cds sequences related with coorFile (longest.trans.info.txt) generated by GetProteinSequence.py")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")

	return parser

def creat_parser_for_calculation_of_codon_density():
	'''argument parser.'''
	usage="usage: %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")


	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=64,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-u","--upstream_position",action="store",type="int",default=None,dest="upstream_position",
			help="The left position (codon level) you want to chose.[upstream position]. default=%default")
	parser.add_option("-d","--downstream_position",action="store",type="int",default=None, dest="downstream_position",
			help="The right position (codon level) you want to chose.[downstream position]. default=%default")
	parser.add_option("-F","--transcript_fasta",action="store",type="string",dest="transcript_fasta",
			help="Input the transcript fasta file used for motif detection and codon density calculation. cds sequences related with coorFile (longest.trans.info.txt) generated by GetProteinSequence.py")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")

	return parser


def create_parser_for_riboDensity_atEachPosition():
	'''argument parser.'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
		help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",dest="bam_files",
		help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
		help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
		help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
		help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
		help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
		help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
		help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-U","--unit_type",action="store",type="string",dest="unit", default="codon",
		help="Unit type we used for metagene analysis. Either 'nt' or 'codon'. default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
		help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. %default=default")

	return parser

def creat_parser_for_specific_region():
	'''argument parser.'''
	usage="usage: python %prog [options]" + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-U","--unit_type",action="store",type="string",dest="unit", default="codon",
			help="Unit type we used for metagene analysis. Either 'nt' or 'codon'. default=%default")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option("-L","--Left_position",action="store",type="int",dest="left_position",
			help="Left position of the interesting region")
	parser.add_option("-R","--Right_position",action="store",type="int",dest="right_position",
			help="Right position of the interesting region")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	return parser

def create_parser_for_frame_density():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	parser.add_option("--plot",action="store",type="string",dest="plot",default='yes',
			help="Output a  plot or not.default=%default")
	return parser

def create_parser_for_RPFdist():
	'''argument parser.'''
	usage="usage: python %prog [options]" + '\n' + __doc__ + "\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-f","--bamListFile",action="store",type="string",default=None,dest="bamListFile",
			help="Bam file list, containing 4 columns.Namely bamFiles,readLength, offSet, bamLegend. '-f' and '-i, -r, -s, -t' parameters are mutually exclusive.default=%default.")
	parser.add_option("-i","--input", action="store",type="string",default=None,dest="bam_files",
			help="Input file(s) in bam format. All files should be split by comma e.g. 1.bam,2.bam,3.bam[required]. '-i' and '-f' are mutually exclusive. default=%default")
	parser.add_option("-c","--coordinateFile",action="store",type="string",dest="coorFile",
			help="The file should contain the coordinate of start and stop codon. Generated by OutputTranscriptInfo.py.[required]")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-r","--specific_reads_length",action="store",type="string",dest="read_length",
			help="Specific the lenght to do analysis, comma split. e.g. '28,29,30'.If use all length set 'All'. Bam files diff length select split by '_' e.g. '28,29,30_ALL_27,28' [required]. '-r' and '-f' are mutually exclusive.")
	parser.add_option("-s","--offset",action="store",type="string",dest="read_offset",
			help="Specific the offset corresponding to read length, comma split. e.g. '12,13,13'. No offset set 0. Bam files diff offset select split by '_' e.g. '12,13,13_0_12,12' [required]. '-s' and '-f' are mutually exclusive.")
	parser.add_option("-t","--bam_file_legend",action="store",type="string",dest="bam_file_legend",
			help="The legend of each bam files, comma split. e.g. 'condition1,condition2,condition3' [required]. '-t' and '-f' are mutually exclusive.")
	parser.add_option("-M","--filter_mode",action="store",type="string",dest="mode",default='counts',
			help="Mode for filtering transcripts. Either 'counts' or 'RPKM'. default=%default.")
	parser.add_option('-S','--select_trans_list',action="store",type='string',dest='in_selectTrans',
			help="Selected transcript list used for metagene analysis.This files requires the first column must be the transcript ID  with a column name.")

	parser.add_option("-l","--minimum_cds_codon",action="store",type="int",default=150,dest="min_cds_codon",
			help="Minimum CDS codon (codon unit). CDS codons smaller than \"minimum_cds_codon\" will be skipped. default=%default")
	parser.add_option("-n","--minimum_cds_counts",action="store",type="int",default=128,dest="min_cds_counts",
			help="Minimum CDS counts. CDS counts smaller than \"min_cds_counts\" will be skipped. default=%default")
	parser.add_option("-m","--minimum_norm_region_counts",action="store",type="int",default=64,dest="min_norm_region_counts",
			help="Minimum counts in normalization region . Counts in norm region smaller than \"minimum_norm_region_counts\" will be skipped. default=%default")
	parser.add_option("-e","--normalize_exclude_codon",action="store",type="int",default=30, dest="norm_exclude_codon",
			help="The region of the normalization for each genes should exclude the first \"normalize_exclude_codon\". default=%default")
	parser.add_option('--id-type',action="store",type="string",dest="id_type",default="transcript_id",
			help="define the id type users input. the default is transcript id, if not, will be transformed into transcript id. default=%default")
	return parser

def create_parser_for_tAI():
	'''argument parser.'''
	usage="usage: python %prog [options]"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input", action="store",type="string",dest="transcriptFiles",
			help="Input file(s) in fasta format. All files should be split by comma e.g. 1.fasta,2.fasta,3.fasta[required]\n. Note: input sequence must be cds sequences which could be generated by GetProteinCodingSequence.py.")
	parser.add_option("-N","--tRNA_copy_numbers",action="store",type="string",dest="copy_numbers",
			help="tRNA copy numbers file.")
	parser.add_option("-o","--otput_prefix",action="store",type="string",dest="output_prefix",
			help="Prefix of output files.[required]")
	parser.add_option("-t","--trans_file_legend",action="store",type="string",dest="trans_file_legend",
			help="The legend of each fasta files, comma split. e.g. 'condition1,condition2,condition3' [required]")
	parser.add_option("-u","--upstream_codon",action="store",type="int",default=0,dest="upstream_codon",
			help="Upstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the downstream codon.default=%default")
	parser.add_option("-d","--downstream_codon",action="store",type="int",default=500, dest="downstream_codon",
			help="Downstream codon corresponding to start codon (codon unit). While corresponding to stop codon, it is the upstream codon.default=%default")
	parser.add_option("--table",action="store",type="int",dest="geneticCode",default=1,
			help="Which genetic code to use. default is the standard code.table = 1, is the standard table, ref: https://www.ncbi.nlm.nih.gov/Taxonomy/Utils/wprintgc.cgi")
	return parser


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-14 17:46:13
@LastEditors: Li Fajin
@LastEditTime: 2019-09-28 17:56:41
@Description: the user could directly input the dataframe format file and plot the polarity without re-do from the bam files

		1) the input file must be python DataFrame format, and has N columns (N represents the number of samples),
		index is the transcript id and each column represents the polarity of a specific sample.The input file was generated by Polarity_calculation.py
		2) the output file could be pdf/png/jpg format
'''

import numpy as np
import pandas as pd
import sys
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import seaborn as sns
from optparse import OptionParser
from collections import defaultdict
from .__init__ import __version__

def create_parser():
	'''argument parser'''
	usage="usage: python %prog [options]" +"\n" + __doc__+"\n"
	parser=OptionParser(usage=usage,version=__version__)
	parser.add_option("-i","--input",action="store",type="string",dest="polarity",help="Input file in txt format.Generated by Polarity_calculation.py")
	parser.add_option("-o","--output",action="store",type="string",dest="output_prefix",help="Prefix of output files.[required]")
	parser.add_option("-f","--format",action="store",type="string",dest="output_format",default='pdf',help="Output file format,'pdf','png' or 'jpg'. default=%default")
	parser.add_option('-g','--group',action="store",type="string",dest="group_name",help="Group name of each group separated by comma. e.g. 'si-control,si-eIF3e'")
	parser.add_option('-r','--replicate',action="store",type="string",dest="replicate_name",help="Replicate name of each group separated by comma. e.g. 'si_3e_1_80S,si_3e_2_80S__si_cttl_1_80S,si_ctrl_2_80S'")
	parser.add_option("-y","--ymax",action="store",type="float",dest="ymax",default=5,help="The max of ylim. default=%default")
	parser.add_option("--mode",action="store",type="string",dest="mode",default='all',help="plot all samples or just mean samples [all or mean].If choose 'all',output all samples as well as mean samples, else just mean samples.default=%default")
	return parser

def Draw_polarity_for_all_samples(data,samples,inOutPrefix,inOutFomat,ymax,text_font={"size":20,"family":"Arial","weight":"bold"},legend_font={"size":20,"family":"Arial","weight":"bold"}):
	"""plot polarity scores"""
	plt.rc('font',weight='bold')
	# colors="bgrcmykwbgrcmykw"
	if len(samples) <=8:
		colors=["b","orangered","green","c","m","y","k","w"]
	else:
		colors=sns.color_palette('husl',len(samples))
	fig=plt.figure(figsize=(5,4))
	ax=fig.add_subplot(111)
	## try to use for loop to re-write the plot function
	for i in np.arange(len(samples)):
		lst=data.iloc[:,i].values
		lst=lst[~np.isnan(lst)]
		sns.distplot(lst,hist=False,rug=False,label=samples[i],color=colors[i])
	ax.set_ylim(-0.1,ymax)
	ax.set_xlabel("Polarity score",fontdict=text_font)
	ax.set_ylabel("Relative gene numbers",fontdict=text_font)
	ax.spines["top"].set_visible(False)
	ax.spines["right"].set_visible(False)
	ax.spines["bottom"].set_linewidth(2)
	ax.spines["left"].set_linewidth(2)
	ax.tick_params(which="both",width=2,length=2)
	plt.legend(loc="best",prop=legend_font)
	plt.tight_layout()
	plt.savefig(inOutPrefix+"_polarity."+inOutFomat,format=inOutFomat)
	plt.close()

def calculate_mean_polarity(data,groups,replicates,output_prefix):
	'''calculate the mean polarity of each group based on different replicates'''
	transcriptList=data.index.values
	labels_dict={}
	data_dict={}
	data_mean_dict=defaultdict(dict)
	for g,r in zip(groups,replicates):
		labels_dict[g]=r.strip().split(',')
	## separate different groups
	for g in groups:
		# data_dict[g]=data.loc[:,labels_dict[g]]
		data_dict[g]=data.reindex(columns=labels_dict[g])

	## calculate the mean polarity between different replicates
	for g in groups:
		for trans in transcriptList:
			index=np.where(~np.isnan(data_dict[g].loc[trans,:]))[0]
			if len(index) == len(labels_dict[g]):
				polarity=np.mean(data_dict[g].loc[trans,data_dict[g].columns[index]])
				data_mean_dict[g][trans]=polarity
			elif len(index) < len(labels_dict[g]) and len(index) > 1:
				polarity=np.mean(data_dict[g].loc[trans,data_dict[g].columns[index]])
				data_mean_dict[g][trans]=polarity
			elif len(index) == 1:
				polarity=np.mean(data_dict[g].loc[trans,data_dict[g].columns[index]])
				data_mean_dict[g][trans]=polarity
			elif len(index) == 0:
				continue
			else:
				raise KeyError("Key error, please check your data input!")

	## transform the dict to a python dataframe
	for g in groups:
		data_mean_dict[g]=pd.DataFrame(data_mean_dict[g],index=[g]).T

	## concatenate different data frame
	data_mean=pd.concat([v for v in data_mean_dict.values()],axis=1,sort=True)
	## write the mean density file
	data_mean.to_csv(output_prefix+"_mean_polarity_dataframe.txt",sep="\t")
	return data_mean


def main():
	'''main function'''
	parser=create_parser()
	(options,args)=parser.parse_args()
	(input,output_prefix,output_format,ymax,groups,replicates,mode)=(options.polarity,options.output_prefix,options.output_format,options.ymax,options.group_name.strip().split(','),options.replicate_name.strip().split('__'),options.mode)
	data=pd.read_csv(input,sep="\t",index_col=0)
	samples=data.columns
	## calculate the mean polarity
	data_mean=calculate_mean_polarity(data,groups,replicates,output_prefix)
	samples_mean=data_mean.columns
	text_font={"size":20,"family":"Arial","weight":"bold"}
	legend_font={"size":10,"family":"Arial","weight":"bold"}
	print("Start plot the polarity...",file=sys.stderr)
	if mode in ['all','All','a','A']:
		Draw_polarity_for_all_samples(data,samples,output_prefix,output_format,ymax,text_font=text_font,legend_font=legend_font)
		Draw_polarity_for_all_samples(data_mean,samples_mean,output_prefix+"_mean",output_format,ymax,text_font=text_font,legend_font=legend_font)
	elif mode in ['mean','Mean','m','M']:
		Draw_polarity_for_all_samples(data_mean,samples_mean,output_prefix+"_mean",output_format,ymax,text_font=text_font,legend_font=legend_font)
	else:
		raise IOError("Please enter a correct --mode parameter [all or mean]")
	print("finished plot the polarity!",file=sys.stderr)

if __name__ == "__main__":
    main()


#!/usr/bin/env python
# -*- coding:UTF-8 -*-
'''
@Author: Li Fajin
@Date: 2019-08-21 19:58:49
@LastEditors: Li Fajin
@LastEditTime: 2019-08-30 17:02:45
@Description: This script is used for extract UTR sequences once given coordinate and transcript sequence.
'''



from .FunctionDefinition import *


def extract_UTRs(transcriptFile,startCodonCoorDict,stopCodonCoorDict,output_prefix):
	'''
	This function is used for extracting UTR sequences of a given sequence.
	'''
	trans_sequence_dict=fastaIter(transcriptFile)
	in_selectTrans=trans_sequence_dict.keys()
	i=0
	with open(output_prefix+"_5UTR.fa",'w') as f1,open(output_prefix+"_CDS.fa",'w') as f2,open(output_prefix+"_3UTR.fa",'w') as f3:
		for trans in in_selectTrans:
			trans_sequence=trans_sequence_dict[trans]
			startCoor=int(startCodonCoorDict[trans])-1 # 0-based
			stopCoor=int(stopCodonCoorDict[trans])-3 # 0-based , the first base of stop codon
			UTR5_sequence=trans_sequence[:startCoor]
			cds_sequence=trans_sequence[startCoor:(stopCoor+3)]
			cds_sequence=trans_sequence[startCoor:(stopCoor+3)]
			UTR3_sequence=trans_sequence[(stopCoor+3):]
			UTR5_length=len(UTR5_sequence)
			cds_length=len(cds_sequence)
			UTR3_length=len(UTR3_sequence)
			if cds_length%3 !=0:
				i+=1
			f1.write("%s%s\n" %(">",str(trans)+" "+str(UTR5_length)))
			f1.write("%s\n" %(UTR5_sequence))
			f2.write("%s%s\n" %(">",str(trans)+" "+str(cds_length)))
			f2.write("%s\n" %(cds_sequence))
			f3.write("%s%s\n" %(">",str(trans)+" "+str(UTR3_length)))
			f3.write("%s\n" %(UTR3_sequence))
	print("Notes: There are " + str(i) +" transcripts whose cds sequence cannot be divided by 3!",file=sys.stderr)

def main():
	parser=create_parser_for_UTR_sequence_extraction()
	(options,args)=parser.parse_args()
	if not options.transcriptFile or not options.coorFile or not options.output_prefix:
		raise IOError("Please reset your parameters!")
	startCodonCoorDict,stopCodonCoorDict=parse_coorFile(options.coorFile)
	print("Start extracting UTR sequences...",file=sys.stderr)
	extract_UTRs(options.transcriptFile,startCodonCoorDict,stopCodonCoorDict,options.output_prefix)
	print("Finish the step of UTR sequences extracting!",file=sys.stderr)

if __name__=="__main__":
	main()